@@ -1,7 +1,7 @@
 #!/usr/bin/r -t
 # -*- mode: R; tab-width: 4 -*-
 #
-# Copyright (C) 2010	Dirk Eddelbuettel and Romain Francois
+# Copyright (C) 2010 - 2011	Dirk Eddelbuettel and Romain Francois
 #
 # This file is part of Rcpp.
 #
@@ -18,11 +18,8 @@
 # You should have received a copy of the GNU General Public License
 # along with Rcpp.  If not, see <http://www.gnu.org/licenses/>.
 
-.setUp <- function(){
-	if( ! exists( ".rcpp.stats", globalenv() ) ){
-		# definition of all the functions at once
-
-		f <- list(
+definitions <- function(){
+    list(
 				  "runit_dbeta" = list(
 				  signature(x = "numeric",
 							a = "numeric", b = "numeric"),
@@ -47,8 +44,14 @@
 				  signature( x = "numeric" ),
 				  '
 				  NumericVector xx(x) ;
-				  return List::create(_["NoLog"] = dunif( xx, 0.0 , 1.0 ),
-									  _["Log"]	= dunif( xx, 0.0, 1.0 , true ));
+				  return List::create(
+                                      _["NoLog_noMin_noMax"] = dunif( xx ),
+                                      _["NoLog_noMax"] = dunif( xx, 0.0 ),
+                                      _["NoLog"] = dunif( xx, 0.0 , 1.0 ),
+									  _["Log"]	= dunif( xx, 0.0, 1.0 , true ),
+									  _["Log_noMax"]	= dunif( xx, 0.0, true )
+                                    //,_["Log_noMin_noMax"]	= dunif( xx, true )
+                  );
 				  ')
 
 				  ,
@@ -56,7 +59,9 @@
 				  '
 				  NumericVector xx(x) ;
 				  return List::create(_["NoLog"] = dgamma( xx, 1.0, 1.0),
-									  _["Log"]	 = dgamma( xx, 1.0, 1.0, true ));
+									  _["Log"]	 = dgamma( xx, 1.0, 1.0, true ),
+									  _["Log_noRate"]	 = dgamma( xx, 1.0, true )
+                  );
 				  ')
 
 				  ,
@@ -73,8 +78,13 @@
 				  signature( x = "numeric" ),
 				  '
 				  NumericVector xx(x) ;
-				  return List::create(_["false"] = dnorm( xx, 0.0, 1.0 ),
-									  _["true"]	 = dnorm( xx, 0.0, 1.0, true ));
+				  return List::create(
+                                      _["false_noMean_noSd"] = dnorm( xx ),
+                                      _["false_noSd"] = dnorm( xx, 0.0  ),
+                                      _["false"] = dnorm( xx, 0.0, 1.0 ),
+									  _["true"]	 = dnorm( xx, 0.0, 1.0, true ),
+									  _["true_noSd"]	 = dnorm( xx, 0.0, true ),
+									  _["true_noMean_noSd"]	 = dnorm( xx, true ));
 				  ')
 
 				  ,
@@ -111,6 +121,19 @@
 									  _["upperLog"]	  = pbinom(xx, n, p, false, true ));
 				  ')
 
+				  ,
+				  "runit_pcauchy" = list(
+				  signature( x = "numeric", location = "numeric", scale = "numeric" ),
+				  '
+				  double loc = as<double>(location);
+				  double scl = as<double>(scale);
+				  NumericVector xx(x) ;
+				  return List::create(_["lowerNoLog"] = pcauchy(xx, loc, scl ),
+									  _["lowerLog"]	  = pcauchy(xx, loc, scl, true, true ),
+									  _["upperNoLog"] = pcauchy(xx, loc, scl, false ),
+									  _["upperLog"]	  = pcauchy(xx, loc, scl, false, true ));
+				  ')
+
 				  ,
 				  "runit_punif" = list(
 				  signature( x = "numeric" ),
@@ -134,6 +157,50 @@
 				  ')
 
 				  ,
+				  "runit_pnf" = list(
+				  signature( x = "numeric" ),
+				  '
+				  NumericVector xx(x) ;
+				  return List::create(_["lowerNoLog"] = pnf( xx, 6.0, 8.0, 2.5, true ),
+									  _["lowerLog"]	  = pnf( xx, 6.0, 8.0, 2.5, true, true ),
+									  _["upperNoLog"] = pnf( xx, 6.0, 8.0, 2.5, false ),
+									  _["upperLog"]	  = pnf( xx, 6.0, 8.0, 2.5, false, true ));
+				  ')
+
+				  ,
+				  "runit_pf" = list(
+				  signature( x = "numeric" ),
+				  '
+				  NumericVector xx(x) ;
+				  return List::create(_["lowerNoLog"] = pf( xx, 6.0, 8.0 ),
+									  _["lowerLog"]	  = pf( xx, 6.0, 8.0, true, true ),
+									  _["upperNoLog"] = pf( xx, 6.0, 8.0, false ),
+									  _["upperLog"]	  = pf( xx, 6.0, 8.0, false, true ));
+				  ')
+
+				  ,
+				  "runit_pnchisq" = list(
+				  signature( x = "numeric" ),
+				  '
+				  NumericVector xx(x) ;
+				  return List::create(_["lowerNoLog"] = pnchisq( xx, 6.0, 2.5, true ),
+									  _["lowerLog"]	  = pnchisq( xx, 6.0, 2.5, true, true ),
+									  _["upperNoLog"] = pnchisq( xx, 6.0, 2.5, false ),
+									  _["upperLog"]	  = pnchisq( xx, 6.0, 2.5, false, true ));
+				  ')
+
+				  ,
+				  "runit_pchisq" = list(
+				  signature( x = "numeric" ),
+				  '
+				  NumericVector xx(x) ;
+				  return List::create(_["lowerNoLog"] = pchisq( xx, 6.0 ),
+									  _["lowerLog"]	  = pchisq( xx, 6.0, true, true ),
+									  _["upperNoLog"] = pchisq( xx, 6.0, false ),
+									  _["upperLog"]	  = pchisq( xx, 6.0, false, true ));
+				  ')
+
+                  ,
 				  "runit_pnorm" = list(signature( x = "numeric" ),
 				  '
 				  NumericVector xx(x) ;
@@ -225,13 +292,15 @@
 			      return wrap(qt( xx, d, lt, lg));
 				  ')
 
-                  ) ## end of list of test function sources
+         )
+}
 
-		signatures <- lapply( f, "[[", 1L )
-		bodies <- lapply( f, "[[", 2L )
-		fx <- cxxfunction( signatures, bodies, plugin = "Rcpp" )
-		getDynLib( fx ) # just forcing loading the dll now
-		assign( ".rcpp.stats", fx, globalenv() )
+.setUp <- function(){
+	if( ! exists( ".rcpp.stats", globalenv() ) ){
+		fun <- Rcpp:::compile_unit_tests(
+		    definitions()
+		)
+	    assign( ".rcpp.stats", fun, globalenv() )
 	}
 }
 
@@ -240,7 +309,8 @@ test.stats.dbeta <- function() {
     vv <- seq(0, 1, by = 0.1)
     a <- 0.5; b <- 2.5
     checkEquals(fx(vv, a, b),
-                list(NoLog = dbeta(vv, a, b),
+                list(
+                     NoLog = dbeta(vv, a, b),
                      Log   = dbeta(vv, a, b, log=TRUE)
                      ),
                 msg = " stats.qbeta")
@@ -248,17 +318,25 @@ test.stats.dbeta <- function() {
 
 test.stats.dbinom <- function( ){
 	fx <- .rcpp.stats$runit_dbinom
-	checkEquals(fx(1:10) ,
-                list( false = dbinom(1:10, 10, .5), true = dbinom(1:10, 10, .5, TRUE ) ),
-                msg = "stats.dbinom" )
+    v <- 1:10
+	checkEquals(fx(v) ,
+                list(
+                    false = dbinom(v, 10, .5),
+                    true = dbinom(v, 10, .5, TRUE )
+                ), msg = "stats.dbinom" )
 }
 
 test.stats.dunif <- function() {
     fx <- .rcpp.stats$runit_dunif
     vv <- seq(0, 1, by = 0.1)
     checkEquals(fx(vv),
-                list(NoLog = dunif(vv),
-                     Log   = dunif(vv, log=TRUE)
+                list(
+                    NoLog_noMin_noMax = dunif(vv),
+                    NoLog_noMax = dunif(vv, 0),
+                    NoLog = dunif(vv, 0, 1),
+                     Log   = dunif(vv, 0, 1, log=TRUE),
+                     Log_noMax   = dunif(vv, 0, log=TRUE)
+                     #,Log_noMin_noMax   = dunif(vv, log=TRUE)  ## wrong answer
                      ),
                 msg = " stats.dunif")
 }
@@ -267,32 +345,42 @@ test.stats.dgamma <- function( ) {
     fx <- .rcpp.stats$runit_dgamma
     v <- 1:4
     checkEquals(fx(v),
-                list( NoLog = dgamma(v, 1.0, 1.0), Log = dgamma(v, 1.0, 1.0, log = TRUE ) ),
-                msg = "stats.dgamma" )
+                list( NoLog = dgamma(v, 1.0, 1.0),
+                      Log = dgamma(v, 1.0, 1.0, log = TRUE ),
+                      Log_noRate = dgamma(v, 1.0, log = TRUE )
+                ), msg = "stats.dgamma" )
 }
 
 
 test.stats.dpois <- function( ){
 	fx <- .rcpp.stats$runit_dpois
-	checkEquals(fx(0:5) ,
-                list( false = dpois(0:5, .5), true = dpois(0:5, .5, TRUE ) ),
-                msg = "stats.dpois" )
+    v <- 0:5
+	checkEquals(fx(v) ,
+                list( false = dpois(v, .5),
+                      true = dpois(v, .5, TRUE )
+                ), msg = "stats.dpois" )
 }
 
 test.stats.dnorm <- function( ) {
     fx <- .rcpp.stats$runit_dnorm
     v <- seq(0.0, 1.0, by=0.1)
     checkEquals(fx(v),
-                list( false = dnorm(v, 0.0, 1.0), true = dnorm(v, 0.0, 1.0, TRUE ) ),
-                msg = "stats.dnorm" )
+                list( false_noMean_noSd = dnorm(v),
+                      false_noSd = dnorm(v, 0.0),
+                      false = dnorm(v, 0.0, 1.0),
+                      true = dnorm(v, 0.0, 1.0, log=TRUE ),
+                      true_noSd = dnorm(v, 0.0, log=TRUE ),
+                      true_noMean_noSd = dnorm(v, log=TRUE )
+                ), msg = "stats.dnorm" )
 }
 
 test.stats.dt <- function( ) {
 	fx <- .rcpp.stats$runit_dt
     v <- seq(0.0, 1.0, by=0.1)
     checkEquals(fx(v),
-                list( false = dt(v, 5), true = dt(v, 5, log=TRUE ) ), # NB: need log=TRUE here
-                msg = "stats.dt" )
+                list( false = dt(v, 5),
+                      true = dt(v, 5, log=TRUE ) # NB: need log=TRUE here
+                ), msg = "stats.dt" )
 }
 
 test.stats.pbeta <- function( ) {
@@ -328,6 +416,20 @@ test.stats.pbinom <- function( ) {
                 msg = " stats.pbinom")
 }
 
+test.stats.pcauchy <- function( ) {
+    fx <- .rcpp.stats$runit_pcauchy
+    location <- 0.5
+    scale <- 1.5
+    vv <- 1:5
+    checkEquals(fx(vv, location, scale),
+                list(lowerNoLog = pcauchy(vv, location, scale),
+                     lowerLog   = pcauchy(vv, location, scale, log=TRUE),
+                     upperNoLog = pcauchy(vv, location, scale, lower=FALSE),
+                     upperLog   = pcauchy(vv, location, scale, lower=FALSE, log=TRUE)
+                     ),
+                msg = " stats.pcauchy")
+}
+
 test.stats.punif <- function( ) {
     fx <- .rcpp.stats$runit_punif
     v <- qunif(seq(0.0, 1.0, by=0.1))
@@ -341,6 +443,54 @@ test.stats.punif <- function( ) {
     # TODO: also borrow from R's d-p-q-r-tests.R
 }
 
+test.stats.pf <- function( ) {
+    fx <- .rcpp.stats$runit_pf
+    v <- (1:9)/10
+    checkEquals(fx(v),
+                list(lowerNoLog = pf(v, 6, 8, lower=TRUE, log=FALSE),
+                     lowerLog   = pf(v, 6, 8, log=TRUE ),
+                     upperNoLog = pf(v, 6, 8, lower=FALSE),
+                     upperLog   = pf(v, 6, 8, lower=FALSE, log=TRUE)
+                     ),
+                msg = "stats.pf" )
+}
+
+test.stats.pnf <- function( ) {
+    fx <- .rcpp.stats$runit_pnf
+    v <- (1:9)/10
+    checkEquals(fx(v),
+                list(lowerNoLog = pf(v, 6, 8, ncp=2.5, lower=TRUE, log=FALSE),
+                     lowerLog   = pf(v, 6, 8, ncp=2.5, log=TRUE ),
+                     upperNoLog = pf(v, 6, 8, ncp=2.5, lower=FALSE),
+                     upperLog   = pf(v, 6, 8, ncp=2.5, lower=FALSE, log=TRUE)
+                     ),
+                msg = "stats.pnf" )
+}
+
+test.stats.pchisq <- function( ) {
+    fx <- .rcpp.stats$runit_pchisq
+    v <- (1:9)/10
+    checkEquals(fx(v),
+                list(lowerNoLog = pchisq(v, 6, lower=TRUE, log=FALSE),
+                     lowerLog   = pchisq(v, 6, log=TRUE ),
+                     upperNoLog = pchisq(v, 6, lower=FALSE),
+                     upperLog   = pchisq(v, 6, lower=FALSE, log=TRUE)
+                     ),
+                msg = "stats.pchisq" )
+}
+
+test.stats.pnchisq <- function( ) {
+    fx <- .rcpp.stats$runit_pnchisq
+    v <- (1:9)/10
+    checkEquals(fx(v),
+                list(lowerNoLog = pchisq(v, 6, ncp=2.5, lower=TRUE, log=FALSE),
+                     lowerLog   = pchisq(v, 6, ncp=2.5, log=TRUE ),
+                     upperNoLog = pchisq(v, 6, ncp=2.5, lower=FALSE),
+                     upperLog   = pchisq(v, 6, ncp=2.5, lower=FALSE, log=TRUE)
+                     ),
+                msg = "stats.pnchisq" )
+}
+
 test.stats.pgamma <- function( ) {
     fx <- .rcpp.stats$runit_pgamma
     v <- (1:9)/10
@@ -462,5 +612,5 @@ test.stats.qt <- function( ) {
 }
 
 # TODO: test.stats.qgamma
-# TODO: test.stats.(dpq)chisq
+# TODO: test.stats.(dq)chisq
 
